//
//  MapViewController.m
//  AntennaField
//
//  Created by Dan on 6/23/13.
//

#import "MapViewController.h"
#import "AntennaAppDelegate.h"
#import "PatternViewController.h"
#import <MapKit/MapKit.h>
#import <CoreLocation/CoreLocation.h>

@interface MapViewController () <CLLocationManagerDelegate, MKMapViewDelegate>
@property (nonatomic, strong) MKMapView *mapView;
@property (nonatomic) double scale;
@property (nonatomic, strong) CLLocationManager *locationManager;
@end

@implementation MapViewController

- (id)initWithMapView:(MKMapView *)mapView {
    self = [super init];
    if (self) {
        self.mapView = mapView;
        self.locationManager.delegate = self;
        self.mapView.delegate = self;
        self.latitude  =  26.20;
        self.longitude = -80.17;
        self.radius    = 20000.0;
        self.scale    = 1.0;
    }
    return self;
}

- (void)setScale:(double)scale {
    _scale = 1.0/scale;
    [self updateMap];
}

- (void)updateMap {
    CLLocation *location = [[CLLocation alloc] initWithLatitude:self.latitude longitude:self.longitude];
    [self zoomToLocation:location radius:self.radius / self.scale];
}

- (CLLocationManager *)locationManager {
    if (!_locationManager) {
        _locationManager = [[CLLocationManager alloc] init];
        _locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters;
        _locationManager.delegate = self;
    }
    return _locationManager;
}

- (void)updateLocation {
    [self.locationManager startUpdatingLocation];
}

#pragma mark - CLLocationManagerDelegate methods

-(void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray *)locations {
    CLLocation *location = [locations lastObject];
    [self zoomToLocation:location radius:2000];

    [self.locationManager stopUpdatingLocation];
}

- (void)zoomToLocation:(CLLocation *)location radius:(CGFloat)radius {
    MKCoordinateRegion region = MKCoordinateRegionMakeWithDistance(location.coordinate, radius * 2, radius * 2);
    [self.mapView setRegion:region animated:NO];
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

@end
