//
//  PatternHelper.m
//
//  Created by Dan Grassi on 7/10/11.
//  Copyright 2011 Zaph, LLC. All rights reserved.
//

#import "PatternHelper.h"
#import "PatternViewController.h"
#import "Utility.h"
#import "PatternModel.h"
#import "PatternView.h"
#import "Tower.h"

@interface PatternHelper ()
@property (nonatomic, strong) PatternView  *patternView;
@property (nonatomic, strong) PatternModel *patternModel;
@end

@implementation PatternHelper

- (id)initWithPatternViewController:(PatternViewController *)patternViewController {
    self = [super init];
    if (self) {
        self.patternView  = patternViewController.patternView;
        self.patternModel = patternViewController.patternModel;
    }
    
    return self;
}

- (void)calculatePatternPath {
    NSMutableArray *pathPoints = [self.patternModel createPatternPoints];
    
    UIBezierPath *bezierPath = [UIBezierPath bezierPath];
	self.patternView.bezierPath = [self addPoints:pathPoints toBezierPath:bezierPath];
    [bezierPath applyTransform:self.patternView.patternTransform];
}

- (void)calculateReferencePath {
    UIBezierPath *bezierPath = [UIBezierPath bezierPath];
    
//    [self appendCircleToBezierPath:bezierPath withScale:1.0];
    [self appendCircleToBezierPath:bezierPath withScale:self.patternModel.fieldRMS];
    [self appendAxesToBezierPath:bezierPath withScale:1.0];
    
    [bezierPath applyTransform:self.patternView.patternTransform];
    self.patternView.bezierPathUnit = bezierPath;
}

- (void)appendCircleToBezierPath:(UIBezierPath *)bezierPath withScale:(CGFloat)scale {
    [bezierPath appendPath:[UIBezierPath bezierPathWithOvalInRect:CGRectMake(-scale, -scale, 2.0*scale, 2.0*scale)]];
}

- (void)appendAxesToBezierPath:(UIBezierPath *)bezierPath withScale:(CGFloat)scale {
    [bezierPath moveToPoint:CGPointMake(-scale, 0.0)];
    [bezierPath addLineToPoint:CGPointMake(scale, 0.0)];
    
    [bezierPath moveToPoint:CGPointMake(0.0, -scale)];
    [bezierPath addLineToPoint:CGPointMake(0.0, scale)];
}

- (UIBezierPath *)addPoints:(NSArray *)points toBezierPath:(UIBezierPath *)bezierPath {
    FieldPoint *fieldPoint = [points objectAtIndex:0];
    CGFloat x = 0 * fieldPoint.magnitude;
    CGFloat y = 1 * fieldPoint.magnitude;
    [bezierPath moveToPoint:CGPointMake(x, y)];
    
	for (fieldPoint in points) {
        x = sinf(fieldPoint.angle + M_PI) * fieldPoint.magnitude;
        y = cosf(fieldPoint.angle + M_PI) * fieldPoint.magnitude;
        [bezierPath addLineToPoint:CGPointMake(x, y)];
	}
    
	return bezierPath;
}

- (void)calculateTowerPoints {
    NSMutableArray *plotPoints = [NSMutableArray array];
    for (Tower *tower in self.patternModel.towers) {
        FieldPoint *plotPoint = [self fieldPointForTower:tower];
        [plotPoints addObject:plotPoint];
    }
    self.patternView.towerPoints = plotPoints;
    Tower *selectedTower = self.patternViewController.selectedTower;
    self.patternView.selectedTowerNumber = [self.patternModel indexOfTower:selectedTower];
}

- (FieldPoint*)fieldPointForTower:(Tower*)tower {
    FieldPoint *point = [FieldPoint pointWithMagnitude:tower.spacing angle:tower.orientation];
    float scale = 0.1;

    CGFloat x =  point.x * scale;
    CGFloat y = -point.y * scale;
    FieldPoint *plotPoint = [FieldPoint pointWithPoint:[self.patternView translatePlotX:x Y:y]];

    return plotPoint;
}

@end
