//
//  PatternView.m
//
//  Created by Dan on 2/18/10.
//  Copyright 2010 Zaph, LLC. All rights reserved.
//

#import "PatternView.h"
#import "Utility.h"
#include <CoreGraphics/CGColorSpace.h>


@interface PatternView ()
@property (nonatomic, assign, readwrite) CGAffineTransform patternTransform;
@property (strong) UIColor *colorPattern;
@property (strong) UIColor *colorUnitCircle;

@property CGPoint offset;
@property CGFloat scale;
@end


@implementation PatternView

- (id)initWithCoder:(NSCoder *)decoder {
    self = [super initWithCoder:decoder];
	if (self) {
        _colorPattern    = [UIColor colorWithHue:120.0/360.0 saturation:0.0 brightness:0.5 alpha:0.33];
        _colorUnitCircle = [UIColor blackColor];
        
        CGFloat size = MIN(self.frame.size.width/2.0, self.frame.size.height/2.0);
        _offset  = CGPointMake(size, size);
        _scale   = size - 4.0;
        
        self.patternTransform = CGAffineTransformIdentity;
        self.patternTransform = CGAffineTransformTranslate(self.patternTransform, _offset.x, _offset.y);
        self.patternTransform = CGAffineTransformScale(self.patternTransform, _scale, _scale);
    }
	return self;
}

-(void)drawRect:(CGRect)rect {
	CGContextRef context = UIGraphicsGetCurrentContext();

    CGFloat pathWidth = 1;
    [self strokeBezierPathWithColor:self.colorUnitCircle pathWidth:pathWidth path:self.bezierPathUnit];
    [self fillBezierPathWithColor:  self.colorPattern    pathWidth:pathWidth path:self.bezierPath];
    [self drawTowersWithContext:context];
}

- (void)strokeBezierPathWithColor:(UIColor *)color pathWidth:(CGFloat)width path:(UIBezierPath *)path {
    [self.colorUnitCircle setStroke];
    [self.bezierPathUnit setLineWidth:width];
    [self.bezierPathUnit stroke];
}

- (void)fillBezierPathWithColor:(UIColor *)color pathWidth:(CGFloat)width path:(UIBezierPath *)path {
    [color setFill];
    [path setLineWidth:width];
    [path fill];
}

- (void)drawTowersWithContext:(CGContextRef)context {
    UIFont *font = [UIFont systemFontOfSize:14];
    UIColor *selectedNumberUIColor   = [UIColor whiteColor];
    UIColor *unSelectedNumberUIColor = [UIColor blackColor];
    UIColor *selectedCircleUIColor   = [UIColor colorWithHue:0.33 saturation:1 brightness:.75 alpha:1];
    UIColor *unSelectedCircleUIColor = [UIColor colorWithHue:0    saturation:0 brightness:.85 alpha:1];

    int i = 0;
    for (FieldPoint *point in self.towerPoints) {
        float x = point.x;
        float y = point.y;
        
        NSString *towerNumber = [NSString stringWithFormat:@"%d", i+1];
        CGSize numberSize = [towerNumber sizeWithFont:font];
        
        CGFloat circleSize = MAX(numberSize.width, numberSize.height) + 2;
        CGRect  circleRect = CGRectMake(x-circleSize/2, y-circleSize/2, circleSize, circleSize);
        
        
        UIColor *towerColor;
        UIColor *numberColor;
        if (self.selectedTowerNumber == i) {
            towerColor  = selectedCircleUIColor;
            numberColor = selectedNumberUIColor;
        }
        else {
            towerColor  = unSelectedCircleUIColor;
            numberColor = unSelectedNumberUIColor;
        }
        
        [towerColor set];
        CGContextFillEllipseInRect(context, circleRect);
        
        CGRect numberRect = CGRectMake(x-numberSize.width/2, y-numberSize.height/2, numberSize.width, numberSize.height);
        [numberColor set];
        [towerNumber drawInRect:numberRect withFont:font];
        
        i += 1;
    }
}

//- (CGPoint)translatePlotPoint:(CGPoint)point {
//    return CGPointMake(point.x * self.scale + self.offset.x, point.y * self.scale + self.offset.y);
//}
- (CGPoint)translatePlotX:(CGFloat)x Y:(CGFloat)y {
    return CGPointMake(x * self.scale + self.offset.x, y * self.scale + self.offset.y);
}

- (CGPoint)translateViewPoint:(CGPoint)point {
    return CGPointMake((point.x - self.offset.x) / self.scale , (point.y - self.offset.y) / self.scale);
}
//- (CGPoint)translateViewX:(CGFloat)x Y:(CGFloat)y {
//    return CGPointMake((x - self.offset.x) / self.scale , (y - self.offset.y) / self.scale);
//}

@end
