//
//  PatternViewController.m
//
//  Created by Dan on 2/18/10.
//  Copyright Zaph, LLC 2010. All rights reserved.
//

#import "PatternViewController.h"
#import "Utility.h"
#import "PatternModel.h"
#import "PatternView.h"
#import "Tower.h"
#import "PatternHelper.h"
#import "MapViewController.h"

@interface PatternViewController ()
@property (nonatomic, strong) IBOutlet UISlider    *fieldSlider;
@property (nonatomic, strong) IBOutlet UITextField *fieldTextField;
@property (nonatomic, strong) IBOutlet UISlider    *phaseSlider;
@property (nonatomic, strong) IBOutlet UITextField *phaseTextField;
@property (nonatomic, strong) IBOutlet UISlider    *spacingSlider;
@property (nonatomic, strong) IBOutlet UITextField *spacingTextField;
@property (nonatomic, strong) IBOutlet UISlider    *orientationSlider;
@property (nonatomic, strong) IBOutlet UITextField *orientationTextField;
@property (nonatomic, strong) IBOutlet UILabel     *fieldRMSLabel;
@property (strong, nonatomic) IBOutlet UIView      *buttonContainerView;
@property (nonatomic, strong) IBOutlet MKMapView   *mapView;

@property (nonatomic) float fieldScale;
@property(nonatomic, strong) MapViewController *mapViewController;

@property(nonatomic, strong) PatternHelper *patternHelper;
@property(nonatomic, strong) Tower *touchTower;

@end


@implementation PatternViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [self addBackgroundToButtons];
    self.patternModel = [[PatternModel alloc] init] ;

    self.fieldScale = sqrt(2.0);
    
    self.patternHelper = [[PatternHelper alloc] initWithPatternViewController:self];
    self.patternHelper.patternViewController = self;
    
    [self addTowerWithField:0.5 phaseDegrees:0 spacingDegrees:0  orientationDegrees:0];
    [self addTowerWithField:0.5 phaseDegrees:90 spacingDegrees:90 orientationDegrees:0];
    self.selectedTower = [self.patternModel towerNumber:1];

    self.mapViewController = [[MapViewController alloc] initWithMapView:self.mapView];
}

- (void)addBackgroundToButtons {
    UIImage *buttomBackgroungImage = [UIImage imageNamed:@"ButtonBackground"];
    UIEdgeInsets capInsets = UIEdgeInsetsMake(8, 8, 8, 8);
    UIImage *resizableImage = [buttomBackgroungImage resizableImageWithCapInsets:capInsets];
    
    for (UIButton *button in self.buttonContainerView.subviews) {
        [button setBackgroundImage:resizableImage forState:UIControlStateNormal];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    [self updateView];
    [self.mapViewController updateMap];

}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return UIInterfaceOrientationIsPortrait(interfaceOrientation);
}

- (void)setSelectedTower:(Tower *)tower {
    _selectedTower = tower;
    [self updateView];
}

- (void)addTowerWithField:(float)field phase:(float)phase spacing:(float)spacing orientation:(float)orientation {
    Tower *tower = [Tower towerWithField:field phase:phase spacing:spacing orientation:orientation ];
    [self.patternModel addTower:tower];
}

- (void)addTowerWithField:(float)field phaseDegrees:(float)phaseDegrees spacingDegrees:(float)spacingDegrees orientationDegrees:(float)orientationDegrees; {
    Tower *tower = [Tower towerWithField:field phaseDegrees:phaseDegrees spacingDegrees:spacingDegrees orientationDegrees:orientationDegrees];
    [self.patternModel addTower:tower];
}

- (IBAction)normalizeFields: (UIButton *)sender {
    [self.patternModel normalizeFields];
    [self updateView];
}

- (void)updateView {
    self.fieldSlider.value = self.selectedTower.field;
    [self fieldSliderChanged:self.fieldSlider];
    self.phaseSlider.value = self.selectedTower.phaseDegrees;
    [self phaseSliderChanged:self.phaseSlider];
    self.spacingSlider.value = self.selectedTower.spacingDegrees;
    [self spacingSliderChanged:self.spacingSlider];
    self.orientationSlider.value = self.selectedTower.orientationDegrees;
    [self orientationSliderChanged:self.orientationSlider];
//    [self.mapViewController updateMap];
}

- (IBAction)fieldSliderChanged:(UISlider *)sender {
    float fieldValue = self.fieldSlider.value;
    self.fieldTextField.text = [NSString stringWithFormat:@"%0.2f", fieldValue];
    self.selectedTower.field = fieldValue;
    [self plotPattern];
}

- (IBAction)fieldValueChanged:(UITextField *)sender {
    self.fieldSlider.value = sender.text.floatValue;
    [self fieldSliderChanged:self.fieldSlider];
}

- (IBAction)phaseSliderChanged:(UISlider *)sender {
    float phaseValue = self.phaseSlider.value;
    self.selectedTower.phase = [Utility radiansFromDegrees:phaseValue];
    self.phaseTextField.text = [NSString stringWithFormat:@"%0.0f", phaseValue];
    [self plotPattern];
}

- (IBAction)phaseValueChanged:(UITextField *)sender {
    self.phaseSlider.value = sender.text.floatValue;
    [self phaseSliderChanged:self.phaseSlider];
}

- (IBAction)spacingSliderChanged:(UISlider *)sender {
    float spacingValue = self.spacingSlider.value;
    self.selectedTower.spacing = [Utility radiansFromDegrees:spacingValue];
    self.spacingTextField.text = [NSString stringWithFormat:@"%0.0f", spacingValue];
    [self plotPattern];
}

- (IBAction)spacingValueChanged:(UITextField *)sender {
    self.spacingSlider.value = sender.text.floatValue;
    [self spacingSliderChanged:self.spacingSlider];
}

- (IBAction)orientationSliderChanged:(id)sender {
    float orientationValue = self.orientationSlider.value;
    self.selectedTower.orientation = [Utility radiansFromDegrees:orientationValue];
    self.orientationTextField.text = [NSString stringWithFormat:@"%0.0f", orientationValue];
    
    [self plotPattern];
}

- (IBAction)orientationValueChanged:(UITextField *)sender {
    self.orientationSlider.value = sender.text.floatValue;
    [self orientationSliderChanged:self.orientationSlider];
}

- (IBAction)removeTower:(id)sender {
    if (self.patternModel.towerCount < 1)
        return;
    
    [self.patternModel removeTower:self.selectedTower];
    NSUInteger towerNumber = MAX(self.patternModel.towerCount-1, 0);
    self.selectedTower = [self.patternModel towerNumber:towerNumber];
    
    [self updateView];
}

- (IBAction)addTower:(UIButton *)sender {
    float newSpacing = self.patternModel.lastTower.spacing + [Utility radiansFromDegrees:90];
    [self addTowerWithField:self.selectedTower.field phase:self.selectedTower.phase spacing:newSpacing orientation:self.selectedTower.orientation];
    self.selectedTower = self.patternModel.lastTower;

    [self updateView];
}

- (void)plotPattern {
    self.fieldScale = sqrtf(2);

#if 0
    [patternHelper calculatePatternPath];
    [self.patternModel  calculateRMSField];
    [patternHelper calculateReferencePath];
    [patternHelper calculateTowerPoints];
#else    
    dispatch_queue_t queue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
    dispatch_group_t group = dispatch_group_create();
    
    dispatch_group_async(group, queue, ^{ [self.patternHelper calculatePatternPath];   });
    dispatch_group_async(group, queue, ^{ [self.patternModel  calculateRMSField];      });
    dispatch_group_async(group, queue, ^{ [self.patternHelper calculateReferencePath]; });
    dispatch_group_async(group, queue, ^{ [self.patternHelper calculateTowerPoints];   });
        
    dispatch_group_wait(group, DISPATCH_TIME_FOREVER);
    dispatch_release(group);
#endif

    [self.mapViewController setScale:self.patternModel.fieldMax];
    self.fieldRMSLabel.text = [NSString stringWithFormat:@"%.3f", self.patternModel.fieldRMS];
    [self.patternView setNeedsDisplay];
}

- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    UITouch *touch = [touches anyObject];
    if (touch.view != self.patternView)
        return;

    CGPoint location = [touch locationInView:touch.view];
    
    CGFloat slop = 20;
    int towerNumber = 0;

    for (Tower *tower in self.patternModel.towers) {
        FieldPoint* point = [self.patternHelper fieldPointForTower:tower];

        NSLog(@"point x: % .3f,  y: % .3f", point.x, point.y);
        CGRect rect = CGRectMake(point.x, point.y, 0, 0);
        rect = CGRectInset(rect, -slop, -slop);
        if (CGRectContainsPoint(rect, location)) {
            self.touchTower = [self.patternModel towerNumber:towerNumber];
            self.selectedTower = self.touchTower;
        }
        towerNumber += 1;
    }
}

- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    UITouch *touch = [touches anyObject];
    if (touch.view != self.patternView)
        return;
    
    float scale = 0.1;
    CGPoint location = [touch locationInView:touch.view];
    CGPoint towerPoint = [self.patternView translateViewPoint:location];
    float x =  towerPoint.x / scale;
    float y = -towerPoint.y / scale;
    [self.touchTower setLocationX:x Y:y];

    [self updateView];
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    UITouch *touch = [touches anyObject];
    if (touch.view != self.patternView)
        return;
    
    self.touchTower = nil;
}

- (void)touchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
    [self touchesEnded:touches withEvent:event];
    [self updateView ];
}

@end
