//
//  Tower.m
//
//  Created by Dan Grassi on 7/4/11.
//  Copyright 2011 Zaph, LLC. All rights reserved.
//

#import "Tower.h"
#import "Utility.h"


@implementation Tower


- (id)initWithField:(float)field phase:(float)phase spacing:(float)spacing orientation:(float)orientation {
    self = [super init];
    if (self) {
        _field       = field;
        _phase       = phase;
        _spacing     = spacing;
        _orientation = orientation;
    }
    
    return self;
}

+ (id)towerWithField:(float)field phase:(float)phase spacing:(float)spacing orientation:(float)orientation {
    return [[Tower alloc] initWithField:field phase:phase spacing:spacing orientation:orientation];
}

+ (id)towerWithField:(float)field phaseDegrees:(float)phase spacingDegrees:(float)spacing orientationDegrees:(float)orientation {
    return [[Tower alloc] initWithField:field phase:[Utility radiansFromDegrees:phase] spacing:[Utility radiansFromDegrees:spacing] orientation:[Utility radiansFromDegrees:orientation]];
}

-(id)copyWithZone:(NSZone*)zone {
	return [Tower towerWithField:self.field phase:self.phase spacing:self.spacing orientation:self.orientation];
}

- (float)phaseDegrees {
    return [Utility degreesFromRadians:self.phase];
}

- (float)spacingDegrees {
    return [Utility degreesFromRadians:self.spacing];
}

- (float)orientationDegrees {
    return [Utility degreesFromRadians:self.orientation];
}

- (void)setLocationX:(float)x Y:(float)y {
    if (fabsf(x) < 0.001)
        x = 0.001;
        
    self.spacing     = hypot(x, y);
    self.orientation = atan(y / x);

    if (x > 0 && y < 0) {
        self.orientation += M_PI *2;
    }
    if (x < 0) {
        self.orientation += M_PI;
    }

//    Zlog(@"Tower: %@,  x: % 3f, y: % 3f", self, x, y);
}

- (NSString *)description {
    return [NSString stringWithFormat:@"Tower - field: %.1f, phase: %.1f, spacing: %.1f, orientation: %.1f",
            self.field, [Utility degreesFromRadians:self.phase], [Utility degreesFromRadians:self.spacing], [Utility degreesFromRadians:self.orientation]];
}
@end
