//
//  Utility.m
//
//  Created by Dan Grassi on 7/4/11.
//  Copyright 2011 Zaph, LLC. All rights reserved.
//

#import "Utility.h"

void Zlog(NSString *format, ...) {
    if (format == nil) {
        printf("nil\n");
        return;
    }
    va_list argList;
    va_start(argList, format);
    // Perform format string argument substitution, reinstate %% escapes, then print
    NSString *s = [[NSString alloc] initWithFormat:format arguments:argList];
    s = [s stringByReplacingOccurrencesOfString:@"%%" withString:@"%%%%"];
    s = [s stringByReplacingOccurrencesOfString:@"\\n" withString:@"\n"];
    printf("%s\n", [s UTF8String]);
    va_end(argList);
}


@implementation Utility

+ (float)radiansFromDegrees:(float)degrees {
    if (degrees == 0.0f)
        return 0.0f;
    else
        return degrees * M_PI / 180;
}

+ (float)degreesFromRadians:(float)radians {
    if (radians == 0.0f)
        return 0.0f;
    else
        return radians * 180 / M_PI;
}

+ (float)normalizeAngle:(float)radians {
    while (radians < 0.0) {
        radians += 2*M_PI;
    }
    while (radians > 2*M_PI) {
        radians -= 2*M_PI;
    }
    
    return radians;
}

@end


@interface FieldPoint ()
@property (nonatomic) float x;
@property (nonatomic) float y;
@end

@implementation FieldPoint 


+ (FieldPoint *) pointWithMagnitude:(float)magnitude angle:(float)angle {
    return [[FieldPoint alloc] initWithMagnitude:magnitude angle:angle];
}

+ (FieldPoint *) pointWithX:(float)x Y:(float)y {
    if (fabsf(x) < 0.001)
        x = 0.001;
    return [[FieldPoint alloc] initWithMagnitude:hypot(x, y) angle:atan(y/x)];
}

+ (FieldPoint *) pointWithPoint:(CGPoint)point {
    if (fabsf(point.x) < 0.001)
        point.x = 0.001;
    return [[FieldPoint alloc] initWithMagnitude:hypot(point.x, point.y) angle:atan(point.y/point.x)];
}

- (id)initWithMagnitude:(float)magnitude angle:(float)angle {
    self = [[FieldPoint alloc] init];
    if (self) {
        _magnitude = magnitude;
        _angle     = angle;
    }
    return self;
}

- (float)x {
    return cosf(self.angle) * self.magnitude;
}

- (float)y {
    return sinf(self.angle) * self.magnitude;
}

- (void)rotatePoint:(float)radians {
    self.angle = [Utility normalizeAngle:self.angle + radians];
}

- (NSString *)description {
    return [NSString stringWithFormat:@"FieldPoint - magnitude: %.3f, angle: %3.0f, x: % .3f, y: % .3f", self.magnitude, [Utility degreesFromRadians:self.angle], self.x, self.y];
}

@end
//
//
//@interface VectorLocation ()
//@property (nonatomic, readwrite) float magnitude;
//@property (nonatomic, readwrite) float angle;
//@end
//
//@implementation VectorLocation
//
//+ (VectorLocation *)pointWithMagnitude:(float)magnitude angle:(float)angle {
//    VectorLocation *vector = [VectorLocation new];
//    if (vector) {
//        vector.magnitude = magnitude;
//        vector.angle     = angle;
//    }
//    return vector;
//}
//
//@end
